;; Generated package description from temp-buffer-browse.el  -*- no-byte-compile: t -*-
(define-package "temp-buffer-browse" "1.5" "temp buffer browse mode" '((emacs "24")) :commit "db6041b0413fdeefb1f1285e1d9c1039c10fbf04" :url "https://elpa.gnu.org/packages/temp-buffer-browse.html" :authors '(("Leo Liu" . "sdl.web@gmail.com")) :maintainer '("Leo Liu" . "sdl.web@gmail.com") :keywords '("convenience"))
